import csv
import matplotlib.pyplot as plt
from iRRAPy import * 

class Row:
    def __init__(self, pred, act):
        self.prediction = int(pred)
        self.label = int(act)
        
    def print(self):
        print(self.prediction, ",", self.label)

def create_data(file,indpred, indac, header = True, sep = ","):
    with open(file) as csv_file:
        csv_reader = csv.reader(csv_file, delimiter = sep)
        ctr = 0
        data = []
        for row in csv_reader:
            if header:
                if ctr == 0:
                    ctr += 1
                    continue
            r = Row(row[indpred], row[indac])
            ctr += 1
            data.append(r)
    
    data.sort(key = lambda x: x.prediction, reverse = True)
    return data
    
def roc(file, indpred, indac, header = True, sep = ","):
    data = create_data(file, indpred, indac, header, sep)
    x = [0]
    y = [0]
    for i in range(len(data)):
        TN = 0
        TP = 0
        FP = 0
        FN = 0
        if i != 0:
            if data[i].prediction == data[i-1].prediction:
                continue
        for j in range(len(data)):
            if data[j].prediction < data[i].prediction:
                if data[j].label == 0:
                    TN += 1
                else:
                    FN += 1
            else:
                if data[j].label == 0:
                    FP += 1
                else:
                    TP += 1
        TPR = TP/(TP+FN)
        FPR = 1 - TN/(TN+FP)
        x.append(FPR)
        y.append(TPR)
    x.append(1)
    y.append(1)
    return [x,y]
    
    
"""Inizio codice"""
primaROC = roc("CurveROC1.csv", 0, 1, sep = ";")
secondaROC = roc("CurveROC2.csv", 0, 1, sep = ";")
rocWMC = roc("berek.csv", 3, 23)
rocLOC = roc("berek.csv", 13, 23)
rocCA = roc("berek.csv", 9, 23)

print("Press enter to continue")
input()
rra(primaROC[0], primaROC[1], 999, 696)
print("Press enter to continue")
input()
rra(primaROC[0], primaROC[1], 999, 696, recall = True, fallout = True)
print("Press enter to continue")
input()
rra(primaROC[0], primaROC[1], 999, 696, recall = True, c_recall = 0.5, 
    phi = True, c_phi = 0.6)
print("Press enter to continue")
input()
rra(secondaROC[0], secondaROC[1], 842, 853, j = True, c_j = 0.3, 
    markedness = True, c_markedness = 0.4, cUnder = "green", 
    main = "Prova diversa")
print("Press enter to continue")
input()
rra(secondaROC[0], secondaROC[1], 842, 853, ncost = True, lamb=[0.5,0.7],
    mu = 0.8, specificity = True, c_specificity = "uni", p_specificity = 0.5,
    col = "red", lw = 2)
print("Press enter to continue")
input()
rra(rocWMC[0], rocWMC[1], 16, 27, j = True, c_j = 0.63)
print("Press enter to continue")
input()
rra_test([rocWMC[0],rocCA[0]],[rocWMC[1],rocCA[1]], 16, 27,  
        fallout = True, c_fallout = "uni", p_fallout = 0.5, precision = True)
print("Press enter to continue")
input()
rra_result1 = rra(rocWMC[0], rocWMC[1], 16, 27, phi = True, npv = True,
                    c_npv = 0.8, print = False, plot = False)
rra_result2 = rra(rocLOC[0], rocLOC[1], 16, 27, phi = True, npv = True, 
                    c_npv = 0.8, print = False, plot = False)
rra_test([rra_result1, rra_result2], col_best = "purple",
        c_under_best = "red", ls_other = "-.", lwd_best = 2)
print("Press enter to continue")
input()
rra_result1 = rra(rocWMC[0], rocWMC[1], 16, 27, nm = True, fm= True, print = False, plot = False)
rra_result2 = rra(primaROC[0], primaROC[1], 999, 696,nm = True, fm = True, print = False, plot = False)
rra_test([rra_result1, rra_result2], lwd_best = 2, lwd_other = 2)

print("Press enter to continue")
input()
rra_result1 = rra(rocWMC[0], rocWMC[1], 16, 27, recall = True, fallout = True, print = False, plot = False)
rra_result2 = rra(primaROC[0], primaROC[1], 999, 696,nm = True, fm = True, print = False, plot = False)
rra_test([rra_result1, rra_result2], lwd_best = 2, lwd_other = 2, add_under_others = True)

print("Press enter to continue")
input()
rra_result1 = rra(rocWMC[0], rocWMC[1], 16, 27, markedness = True, 
                    c_markedness = 0.2, ncost = True, lamb = [0.4], mu = 1, print = False, plot = False)
rra_result2 = rra(rocLOC[0], rocLOC[1], 16, 27, markedness = True, 
                    c_markedness = 0.2, ncost = True, lamb = [0.4], mu = 1, print = False, plot = False)
rra_result3 = rra(rocCA[0], rocCA[1], 16, 27, markedness = True, 
                    c_markedness = 0.2, ncost = True, lamb = [0.4], mu = 1, print = False, plot = False)
rra_average([rra_result1, rra_result2, rra_result3])
print("Press enter to continue")
input()
rra_result1 = rra(rocWMC[0], rocWMC[1], 16, 27, markedness = True, 
                    c_markedness = 0.2, ncost = True, lamb = [0.4], mu = 1, print = False, plot = False)
rra_result2 = rra(rocLOC[0], rocLOC[1], 16, 27, markedness = True, 
                    c_markedness = 0.2, ncost = True, lamb = [0.4], mu = 1, print = False, plot = False)
rra_result3 = rra(rocCA[0], rocCA[1], 16, 27, markedness = True, 
                    c_markedness = 0.2, ncost = True, lamb = [0.4], mu = 1, print = False, plot = False)
a = rra_test([rra_result1,rra_result2,rra_result3], legend = True, 
        lwd_other = 2, main = "Test multiple RRAs", add_under_others = True)
print(a)
print("Press enter to continue")
input()
a = rra_test([rocWMC[0], rocCA[0], rocLOC[0]], [rocWMC[1], rocCA[1], rocLOC[1]], 16, 27,
            legend = True, ncost = True, lamb = [0.4], markedness = True, 
            c_markedness = 0.2, col_best = "red", main = "Altra prova", 
            xlab = "",ylab = "", add_under_others = True)
print(a)
print("Press enter to continue")
input()
rra_result1 = rra(primaROC[0], primaROC[1], 999, 696, ncost = True, 
                    lamb = [0,0.2], mu = 0.8, print = False, plot = False)
rra_result2 = rra(secondaROC[0], secondaROC[1], 842, 853, ncost = True,
                    lamb = [0,0.2], mu = 0.8, print = False, plot = False)
rra_result3 = rra(rocCA[0], rocCA[1], 16, 27, ncost = True, 
                    lamb = [0,0.2], mu = 0.8, print = False, plot = False)
a = rra_test([rra_result1,rra_result2,rra_result3], add_under_others = True, 
        legend = True)
print(a)
print("Press enter to continue")
input()
rra(rocWMC[0], rocWMC[1], 16, 27, ncost = True, lamb = [0.4, 0.6], mu = 0.8)
print("Press enter to continue")
input()
rra(primaROC[0], primaROC[1], 999, 696, ncost = True, lamb = [0.7, 0.9], mu = 0.7)
print("Press enter to continue")
input()
rra(primaROC[0],primaROC[1],999,696,phi=True,j=True,c_j=0.35,nm=True,c_nm=0.65)
print("Press enter to continue")
input()
rra(rocWMC[0],rocWMC[1],16,27,fallout=True,c_fallout=0.4)
print("The End. Press enter to close")
input()