from warnings import warn
import iRRAPy.errors as err
import iRRAPy.utilRRAgeometry as geo
from warnings import simplefilter

simplefilter("always", UserWarning)
warning_message1 = "\nThe selected reference value for "
warning_message2 = (" represents a non existent RoI. Unless a perfect ROC"
        "curve is being considered, the resulting RRA will be equal to zero")

class Border:
    """ This record contains the slope and intercept value of a piece of the
        RoI border. If it is a vertical line, slope will be -1 and intercept
        represents its x value. If it is a curve, slope will be infinite.
        rra_zero indicates if Border's values represents a
        non-existent RoI"""
    def __init__(self, slope, intercept, rra_zero):
        self.slope = slope
        self.intercept = intercept
        self.rra_zero = rra_zero


def checkC(c):
    return ((isinstance(c, str) and (c != "pop" and c != "uni")) or
            (isinstance(c, (int,float)) and (c < 0 or c > 1)))

""" These functions allow other functions to create Borders objects.
    These are not meant to be used by the user"""

def precision_border(c_precision, p_precision, k):
    if checkC(c_precision):
        raise err.InputValueError(("\"c_precision\" value must be between"
                                    " 0 and 1, \"uni\" or \"pop\""))
    
    rra_zero = False
    if isinstance(c_precision, str):
        if (c_precision == "uni" and (not isinstance(p_precision, (int,float))
                                      or p_precision < 0 or p_precision > 1)):
            raise err.InputValueError(("p(m) for Precision(uni) must"+
                                   " be between 0 and 1"))
        slope = 1
        intercept = 0
    else:
        if c_precision == 1:
            warn(warning_message1 + "precision" + warning_message2)
            slope = -1
            intercept = 0
            rra_zero = True
        else:
            slope = (c_precision*k)/(1-c_precision)
            intercept = 0
    result = Border(slope, intercept, rra_zero)
    return result

def recall_border(c_recall, p_recall, k):
    if checkC(c_recall):
        raise err.InputValueError(("\"c_recall\" value must be between 0"
                               " and 1, \"uni\" or \"pop\""))
    
    rra_zero = False
    if isinstance(c_recall, str):
        if c_recall == "pop":
            if k == 0:
                warn(("\nAN/AP is equal to zero. Therefore, " +
                      warning_message1 + "recall" + warning_message2))
                rra_zero = True
            slope = 0
            intercept = 1/(1+k)
        else:
            if (not isinstance(p_recall, (int,float)) or
                (p_recall < 0 or p_recall > 1)):
                raise err.InputValueError(("p(m) for Recall(uni) must"+
                                       " be between 0 and 1"))
            else:
                if p_recall == 1:
                    warn((warning_message1 + "p(m) for recall" +
                          warning_message2))
                    rra_zero = True
                slope = 0
                intercept = p_recall
    else:
        if c_recall == 1:
            warn((warning_message1 + "recall" + warning_message2))
        slope = 0
        intercept = c_recall

    result = Border(slope, intercept, rra_zero)
    return result

def fm_border(c_fm, p_fm, k):
    if checkC(c_fm):
        raise err.InputValueError(("\"c_fm\" value must be between 0 and 1"
                               " (0 escluded), \"uni\" or \"pop\""))

    rra_zero = False
    if isinstance(c_fm, str):
        if c_fm == "pop":
            if k == 0:
                warn(("\nAN/AP is equal to zero. Therefore, " +
                        warning_message1 + "FM" + warning_message2))
                rra_zero = True
            slope = k/(2*k + 1)
            intercept = 1/(2*k + 1)
        else:
            if (not isinstance(p_fm, (int,float)) or (p_fm < 0 or p_fm > 1)):
                raise err.InputValueError(("p(m) for FM(uni) must"+
                                        " be between 0 and 1"))
            if p_fm == 1 and k == 0:
                warn((warning_message1 + "p(m) for FM, having AN/AP = 0" +
                        ", " + warning_message2))
                rra_zero = True
            slope = (p_fm*k)/((p_fm*k)+1)
            intercept = p_fm/((p_fm*k)+1)
    else:
        if c_fm == 1:
            warn(warning_message1 + "FM" + warning_message2)
            rra_zero = True
        slope = (c_fm*k)/(2-c_fm)
        intercept = c_fm/(2-c_fm)

    result = Border(slope, intercept, rra_zero)
    return result

def npv_border(c_npv, p_npv, k):
    if checkC(c_npv):
        raise err.InputValueError(("\"c_npv\" value must be between 0 and 1"
                               ", \"uni\" or \"pop\""))
    rra_zero = False
    if isinstance(c_npv, str):
        if (c_npv == "uni" and (not isinstance(p_npv, (int,float)) or
                                (p_npv < 0 or p_npv > 1))):
            raise err.InputValueError(("p(m) for NPV(uni) must"+
                                        " be between 0 and 1"))
        slope = 1
        intercept = 0
    else:
        if c_npv == 0 or c_npv == 1:
            warn(warning_message1 + "NPV" + warning_message2)
            slope = -1
            intercept = -1
            rra_zero = True
        else:
            if k == 0:
                warn(("\nAN/AP is equal to zero. Therefore, the NPV border " +
                      warning_message2))
                rra_zero = True
            slope = ((1-c_npv)*k)/c_npv
            intercept = 1-(((1-c_npv)*k)/c_npv)

    result = Border(slope, intercept, rra_zero)
    return result

def specificity_border(c_specificity, p_specificity, k):
    if checkC(c_specificity):
        raise err.InputValueError(("\"c_specifdicty\" value must be between 0"
                                   " and 1, \"uni\" or \"pop\""))
    rra_zero = False
    if isinstance(c_specificity, str):
        if c_specificity == "pop":
            slope = -1
            intercept = 1/(1+k)
        else:
            if (not isinstance(p_specificity, (int,float)) or
                (p_specificity < 0 or p_specificity > 1)):
                raise err.InputValueError(("p(m) for Specificity(uni) "
                                           "must be between 0 and 1"))
            if p_specificity == 0:
                warn((warning_message1 + "p(m) for specificty" +
                      warning_message2))
                rra_zero = True
            slope = -1
            intercept = p_specificity
    else:
        if c_specificity == 1:
            warn(warning_message1 + "specificity" + warning_message2)
            rra_zero = True
        slope = -1
        intercept = 1-c_specificity

    result = Border(slope, intercept, rra_zero)
    return result

def fallout_border(c_fallout, p_fallout, k):
    if checkC(c_fallout):
        raise err.InputValueError(("\"c_fallout\" value must be between 0"
                                   " and 1, \"uni\" or \"pop\""))
    rra_zero = False
    if isinstance(c_fallout, str):
        if c_fallout == "pop":
            slope = -1
            intercept = 1/(1+k)
        else:
            if (not isinstance(p_fallout, (int,float)) or
                (p_fallout < 0 or p_fallout > 1)):
                raise err.InputValueError(("p(m) for Specificity(uni) "
                                           "must be between 0 and 1"))
            if p_fallout == 0:
                warn(warning_message1 + "p(m) for fall-out" + warning_message2)
                rra_zero = True
            slope = -1
            intercept = p_fallout
    else:
        if c_fallout == 0:
            warn(warning_message1 + "fall-out" + warning_message2)
            rra_zero = True
        slope = -1
        intercept = c_fallout

    result = Border(slope, intercept, rra_zero)
    return result

def nm_border(c_nm, p_nm, k):
    if checkC(c_nm):
        raise err.InputValueError(("\"c_nm\" value must be between 0 and 1 "
                                   "(0 escluded), \"uni\" or \"pop\""))

    rra_zero = False
    if isinstance(c_nm, str):
        if c_nm == "pop":
            slope = k+2
            intercept = -1
        else:
            if (not isinstance(p_nm, (int,float)) or
                (p_nm < 0 or p_nm > 1)):
                raise err.InputValueError(("p(m) for NM(uni) "
                                           "must be between 0 and 1"))
            if p_nm == 1:
                warn(warning_message1 + "p(m) for NM" + warning_message2)
                slope = -1
                intercept = -1
                rra_zero = True
            else:
                slope = (1-p_nm+k)/(1-p_nm)
                intercept = -(p_nm*k)/(1-p_nm)
    else:
        if c_nm == 0:
            warn(warning_message1 + "NM" + warning_message2)
            slope = -1
            intercept = -1
            rra_zero = True
        else:
            if k == 0:
                warn(("\nAN/AP is equal to zero. Therefore,"
                      " the NM border" + warning_message2))
                rra_zero = True
            slope = ((2-c_nm)*k)/c_nm
            intercept = 1-((2*k*(1-c_nm))/c_nm)

    result = Border(slope, intercept, rra_zero)
    return result

def j_border(c_j):
    if  not isinstance(c_j, (int,float)) or (c_j < -1 or c_j > 1):
        raise err.InputValueError(("\"c_j\" value must be between -1 and 1"))

    rra_zero = False
    if c_j == 1:
        warn(warning_message1 + "J" + warning_message2)
        rra_zero = True
    slope = 1
    intercept = c_j

    result = Border(slope, intercept, rra_zero)
    return result

def ncost_border(lamb, mu, k):
    if  not isinstance(lamb, (int,float)) or (lamb < 0 or lamb > 1):
        raise err.InputValueError("lambda value must be between 0 and 1")

    rra_zero = False
    if not isinstance(mu, (int,float)) or (mu < 0 or mu > 1):
        raise err.InputValueError("mu value must be between 0 and 1")
    if mu == 0:
        warn(warning_message1 + "mu" + warning_message2)
        slope = -1
        intercept = -1
        rra_zero = True
    elif lamb == 0:
        slope = -1
        intercept = mu*(1/(1+k))
    else:
        if k == 0:
            warn(("AN/AP is equal to zero. Therefore,"
                  " the NC border" + warning_message2))
            rra_zero = True
        slope = ((1-lamb)*k)/lamb
        intercept = 1-((mu*k)/(lamb*(1+k)))

    result = Border(slope, intercept, rra_zero)
    return result

"""These functions return a list of points that represents the curve"""

def phi_border(phi, k, vx):
    vx_def = []
    vy = []
    for j in range(len(vx)):
        x = vx[j]
        y = geo.intersect_phi_vertical_line(phi, k, x)
        if y > 1 and (len(vy) == 0 or vy[j-1] < 1):
            if len(vy) == 0:
                vx_def.append(x)
                vy.append(1)
            else:
                top_point=geo.intersect_segments(0,1,1,1,vx[j-1],vy[j-1],x,y)
                vx_def.append(top_point.x)
                vy.append(top_point.y)
        elif y > 1 and vy[j-1] >= 1:
            vx_def.append(x)
            vy.append(1)
        else:
            vx_def.append(x)
            vy.append(y)
    result = [vx,vy]
    return result
    
def markedness_border(c_markedness, k, vx):
    vx_def = []
    vy = []
    for j in range(len(vx)):
        x = vx[j]
        y = geo.intersect_markedness_vertical_line(c_markedness, k, x)
        if y > 1 and (len(vy) == 0 or vy[j-1] < 1):
            if len(vy) == 0:
                vx_def.append(x)
                vy.append(1)
            else:
                top_point=geo.intersect_segments(0,1,1,1,vx[j-1],vy[j-1],x,y)
                vx_def.append(top_point.x)
                vy.append(top_point.y)
        elif y > 1 and vy[j-1] >= 1:
            vx_def.append(x)
            vy.append(1)
        else:
            vx_def.append(x)
            vy.append(y)
    result = [vx,vy]
    return result

