import matplotlib.pyplot as plt

class RRAresult:
    """ This object contains all the elements needed to print and plot
        the ROC curve, the Region of Interest and the RRA value.
        In paricular: the x and y coordinates of the ROC curve, the RoI
        and the RoI under the curve, the RoI area and area under the curve,
        all the metrics used as reference and their value"""
    def __init__(self, rocx, rocy, roix, roiy, roiunderx, roiundery,
                 aroi, aunder, metrics, metricsvalues):
        self.roc_x = rocx
        self.roc_y = rocy
        self.roi_x = roix
        self.roi_y = roiy
        self.roi_under_x = roiunderx
        self.roi_under_y = roiundery
        self.area_roi = aroi
        self.area_under = aunder
        self.metrics = metrics
        self.metrics_values = metricsvalues

    """ Compute the RRA value. Call: obj.getRRA()"""
    def getRRA(self):
        return self.area_under / self.area_roi

    """ Print the RRAresult object. It indicates every reference values
        considered and then he print the resulting RRA value.
        Call: obj.print()"""
    def print(self):
        if not self.metrics and self.getRRA()!=0 :
            print("The AUC of the ROC curve is equal to", self.getRRA())
        elif not self.metrics and self.getRRA()==0:
            print("The Region of Interest (RoI) is non-existent.",
                  "The ROC curve is not perfect (AUC < 1) so the Ratio of",
                  "Relevant Areas (RRA) is equal to zero")
        else:
            print("The generated Region of Interest (RoI) represents the",
                  "points in the ROC space that have:")
            nc_done = 0
            for i in range(len(self.metrics)):
                if self.metrics[i] != "NC1" and self.metrics[i] != "NC2":
                    met = self.metrics[i]
                    if met == "NC":
                        met = "Normalized Cost"
                    mval = self.metrics_values[i]
                    if (mval != "pop" and not isinstance(mval,(int,float)) and
                        met != "Normalized Cost") :
                        mval = "a uniform random policy (\"uni\") with " + mval
                    elif mval == "pop":
                        mval = ("a uniform random policy with p(m) = AP/n"
                                "(\"pop\")")
                    if i == (len(self.metrics)-1):
                        print("A better ", met, " value than ", mval,".",sep="")
                    else:
                        print("A better ", met, " value than ", mval,",",sep="")
                else:
                    if not nc_done:
                        nc_done = 1
                        met = "Normalized Cost"
                        mval = self.metrics_values[i]
                        mval = "a maximum NC value (mu*NC(pop)) with " + mval
                        if i == (len(self.metrics)-1) or len(self.metrics)==2 :
                            print("A better ", met, " value than ",
                                  mval,".",sep="")
                        else:
                            print("A better ", met, " value than ",
                                  mval,",",sep="")
            print("The Ratio of Relevant Areas (RRA) is equal to",self.getRRA())

    def plot(self, cROI = "grey", cUnder = "c", col="k",
            xlab = "FP/AN", ylab = "TP/AP", main = "ROC curve and RoI", 
            xlim = [0,1], ylim = [0,1], asp = 'auto',
            ls = "-", lw = 1, ADD = False, diagonal = True, show = True,
            label = "ROC curve"):
        if not ADD:
            plt.close()
        plt.fill(self.roi_x, self.roi_y, color = cROI)
        for i in range(len(self.roi_under_x)):
            plt.fill(self.roi_under_x[i], self.roi_under_y[i], color = cUnder)
        plt.plot(self.roc_x, self.roc_y, color = col, linestyle = ls,
            linewidth = lw, label = label)
        if diagonal:
            plt.plot([0,1],[0,1], color = "grey", linestyle = '-', 
                                                            linewidth = 0.5)
        if not ADD:
            plt.gca().set_aspect(asp)
            plt.axis([*xlim,*ylim])
            plt.xlabel(xlab)
            plt.ylabel(ylab)
            plt.title(main)
        if show:
            plt.show()