import iRRAPy.RRAresult as rres
from warnings import warn
from iRRAPy.rra import rra
import matplotlib.pyplot as plt
from warnings import simplefilter
import matplotlib.colors as mcolors
import random

simplefilter("always", UserWarning)

""" It will select the ROC curve with the best RRA value from a list
of RRAresult objects or two lists with the coordinates of the ROC curves.
In this case the user has to also add other parameters from the rra() 
function"""
def rra_test(list1, list2 = [], AP=0, AN=0, plot = True, ADD = False,
            show = True, col_best = "k", c_under_best = "c", croi_best="grey",
            lwd_best = 2, ls_best = "-", lwd_other = 1, ls_other = "--",
            add_under_others = False, legend = False, **args):
    res = []
    if not isinstance(list1[0], rres):
        list1 = rra_test_roc(list1, list2, AP, AN, **args)
    plot_args = dict()
    for a in args:
        if (a != 'xlab' and a != 'ylab' and a != 'main' and a != 'xlim' and
            a != 'ylim' and a != 'asp' and a != 'diagonal'):
                continue
        else:
            plot_args[a] = args[a]
    res = rra_test_result(list1, plot, ADD, col_best,c_under_best,croi_best,
                    lwd_best,ls_best,lwd_other,ls_other,add_under_others,
                    legend, **plot_args)
    if show:
        plt.show()
    return res

def rra_test_result(rra_results,plot, ADD, col_best,c_under_best,croi_best,
                    lwd_best,
                    ls_best,lwd_other,ls_other,add_under_others,legend,
                    **args):
    ff = rra_results[0]
    first = ff.roi_x
    first_y = ff.roi_y
    n = len(first)
    for x in rra_results:
        if n != len(x.roi_x) or first != x.roi_x or first_y != x.roi_y:
            if (len(ff.metrics) != len(x.metrics) or ff.metrics != x.metrics or
                                        ff.metrics_values != x.metrics_values):
                warn("\nThe Regions of Interest considered have been made"
                        " by different performance metrics or different"
                        " metrics' values as references")
            else:
                warn("\nThe ROC curves have been made using different "
                        "data sets (i.e. two different AN/AP values)")
            break
    
    clasf = [0]
    maxRRA = ff.getRRA()
    indMax = 0
    all_the_same = True
    for x in range(1,len(rra_results)):
        if rra_results[x].getRRA() > maxRRA:
            maxRRA = rra_results[x].getRRA()
            indMax = x
        if rra_results[x].getRRA() != rra_results[x-1].getRRA():
            all_the_same = False
        for ii in range(len(clasf)):
            if rra_results[clasf[ii]].getRRA() < rra_results[x].getRRA():
                clasf.insert(ii, x)
                break
            else:
                if ii == len(clasf)-1:
                    clasf.append(x)
    if all_the_same:
        print("All the ROC curves have the same RRA value")
        if plot:
            lwd_best = 1
            ls_other = ls_best
            lwd_other = lwd_best
    else:
        print("The ROC curve number ",indMax," (color = ", col_best, ") has "
                                        "the best RRA value", sep = "")
    if plot:
        if not ADD:
            plt.close()
            rra_results[indMax].plot(ADD = False, show = False,cROI=croi_best,
                                    cUnder = c_under_best, col = col_best,  
                                    ls = ls_best, lw=lwd_best, label = "Best",
                                    **args)
        else:
            rra_results[indMax].plot(ADD = True, show = False,
                                    cUnder = c_under_best, col = col_best, 
                                    ls = ls_best, lw=lwd_best, label = "Best", 
                                    **args)
        """Colors"""
        a = mcolors.TABLEAU_COLORS
        colores = []
        for i in a:
            colores.append(a[i])
        random.shuffle(colores)
        for x in clasf:
            if x == indMax:
                continue
            colind = x%len(colores)
            colore = colores[colind]
            if (colore == str(mcolors.to_hex(col_best)) and 
                colore == str(mcolors.to_hex(c_under_best)) and
                colore == str(mcolors.to_hex(croi_best))):
                colore = colores[(colind+1)%len(colores)]
            rrares = rra_results[x]
            if add_under_others:
                if len(rrares.roi_under_x) > 0:
                    for l in range(len(rrares.roi_under_x)):
                        plt.fill(rrares.roi_under_x[l], 
                            rrares.roi_under_y[l],color=colore,alpha=0.5,
                            ls = ls_other)
            plt.plot(rrares.roc_x,rrares.roc_y,color=colore,ls=ls_other,
                        lw=lwd_other, label = ("ROC " + str(x)))
        if legend:
            plt.legend(loc = "lower right")
        plt.draw()
    return clasf
    
def rra_test_roc(roc_xs, roc_ys, AP, AN, **args):
    list = []
    for i in range(len(roc_xs)):
        list.append(rra(roc_xs[i],roc_ys[i],AP,AN,**args,plot=False,print=False))
    return list