import iRRAPy.utilRRAgeometry as geo

"""Methods used to find the coordinates of a ROC curve"""

"""Returns the list of coordinates of a ROC curve in a certain range"""
def partial_roc_curve(roc_x, roc_y, xsx, xdx):
    if xsx < xdx:
        vx = []
        vy = []
        for i in range(len(roc_x)-1):
            x1 = roc_x[i]
            y1 = roc_y[i]
            x2 = roc_x[i+1]
            y2 = roc_y[i+1]
            if x1 < xsx and x2 > xsx:
                yint = geo.intersect_vertical_line_segment(xsx, x1, y1, 
                           x2, y2)
                vx.append(xsx)
                vy.append(yint)
                if x2 > xdx:
                    yint = geo.intersect_vertical_line_segment(xdx, x1, y1, 
                           x2, y2)
                    vx.append(xdx)
                    vy.append(yint)
            elif x1 >= xsx and x1 <= xdx:
                vx.append(x1)
                vy.append(y1)
                if x1 < xdx and x2 > xdx:
                    yint = geo.intersect_vertical_line_segment(xdx, x1, y1, 
                           x2, y2)
                    vx.append(xdx)
                    vy.append(yint)
        
        result = [vx, vy]
        return result
    else:
        return None
  
"""Intersection between a ROC curve and a line within an interval"""
def intersect_roc_curve_line(roc_x, roc_y, slope, intercept, xsx, xdx):
    result = []
    if xsx < xdx:
        vv = partial_roc_curve(roc_x, roc_y, xsx, xdx)
        vx = vv[0]
        vy = vv[1]
        
        bb = False
        for i in range(len(vx)-1):
            xa = vx[i]
            ya = vy[i]
            xb = vx[i+1]
            yb = vy[i+1]
            y1 = slope*xa + intercept
            y2 = slope*xb + intercept
            
            if ((y1 <= ya and y2 >= yb) or (y1 >= ya and y2 <= yb)):
                if y2 == yb:
                    if (i+2) < len(vx):
                        xc = vx[i+2]
                        yc = vy[i+2]
                        y3 = slope*xc + intercept
                        if yc >= y3:
                            bb = True
                        """This is when the line intersect the ROC in just 
                        one point. In this case the point will be used twice
                        in the next iteration so the rra function works 
                        properly"""
                    continue
                else:
                    pint = geo.intersect_line_segment(slope, intercept, 
                                                        xa, ya, xb, yb)
                    result.append(pint)
                    if bb:
                        result.append(pint)
                        bb = False
        return result
    else:
        return result
        
"""Intersection between a ROC curve and a vertical line"""
def intersect_roc_curve_vertical_line(roc_x, roc_y, x):
    result = []
    for i in range(len(roc_x)-1):
        if roc_x[i] == x:
            result.append(geo.Intersection(x, roc_y[i]))
        elif roc_x[i] < x and roc_x[i+1] > x:
            yint = geo.intersect_vertical_line_segment(x, roc_x[i], roc_y[i],
                                                    roc_x[i+1], roc_y[i+1])
            result.append(geo.Intersection(x, yint))
        elif roc_x[i] > x:
            break
    return result
        
"""Intersection between a ROC curve and a phi within a certain interval"""
def intersect_roc_curve_phi(roc_x, roc_y, phi, k, xsx, xdx):
    result = []
    if xsx < xdx:
        vv = partial_roc_curve(roc_x, roc_y, xsx, xdx)
        vx = vv[0]
        vy = vv[1]
        
        for i in range(len(vx)-1):
            xa = vx[i]
            ya = vy[i]
            xb = vx[i+1]
            yb = vy[i+1]
            y1 = geo.intersect_phi_vertical_line(phi, k, xa)
            y2 = geo.intersect_phi_vertical_line(phi, k, xb)
            
            if not(y1 > ya and y2 > yb):
                pint = geo.intersect_phi_segment(phi, k, xa, xb, ya, yb)
                if not pint is None:
                    for p in pint:
                        result.append(geo.Intersection(p.x, p.y))
        return result
    else:
        return result

"""Intersection between a ROC curve and a markedness curve within a certain
    interval"""
def intersect_roc_curve_markedness(roc_x, roc_y, c_markedness, k, xsx, xdx):
    result = []
    if xsx < xdx:
        vv = partial_roc_curve(roc_x, roc_y, xsx, xdx)
        vx = vv[0]
        vy = vv[1]
        
        for i in range(len(vx)-1):
            xa = vx[i]
            ya = vy[i]
            xb = vx[i+1]
            yb = vy[i+1]
            y1 = geo.intersect_markedness_vertical_line(c_markedness, k, xa)
            y2 = geo.intersect_markedness_vertical_line(c_markedness, k, xb)
            
            if not(y1 > ya and y2 > yb):
                pint = geo.intersect_markedness_segment(c_markedness, k, xa,
                                                    xb, ya, yb)
                if not pint is None:
                    for p in pint:
                        result.append(geo.Intersection(p.x, p.y))

        return result
    else:
        return result