from scipy.integrate import quad
from math import sqrt

""" Methods used by other functions of this package to compute areas in the
    ROC space. They are not meant to be used directly """

""" Compute the area of a quadrilateral given the coordinates. The poligon has
    the two opposite sides perpendicular to the x-axis and x1 < x2 """
def quadrilateral_area(x1, y1_up, y1_down, x2, y2_up, y2_down):
    if x2 > x1:
        higher_up_y = max(y1_up, y2_up)
        lower_up_y = min(y1_up, y2_up)
        higher_down_y = max(y1_down, y2_down)
        lower_down_y = min(y1_down, y2_down)
        h = x2-x1

        area = h*(higher_up_y - lower_down_y)
        area -= h*(higher_up_y - lower_up_y)/2
        area -= h*(higher_down_y - lower_down_y)/2
        return area
    else:
        return 0

""" Compute the area of a triangle given the coordinates of its three points"""
def triangle_area(x1, y1, x2, y2, x3, y3):
    area = abs((x1-x3)*(y2-y1) - (x1-x2)*(y3-y1))/2
    return area
    
""" Compute the area under the markedness curve"""
def area_under_markedness_interval(c_markedness, k, x1, x2):
    u = c_markedness
    a = u*(k**2)
    b = 2*u*k
    c = u
    d = -k*(u*k + u + 1)
    e = -(u*k - k + u)
    if u == 0:
        def func(x):
            return x
    else:
        def func(x):
            return (-(b*x+e)+sqrt(((b*x+e)**2)-4*c*(a*(x**2)+d*x)))/(2*c)
    integration = quad(func, x1, x2)
    return integration[0]
    
""" Compute the area under the phi curve"""
def area_under_phi_interval(phi, k, x1, x2):
    s = phi**2
    a = k+s*(k**2)
    b = 2*k*(s-1)
    c = s+k
    d = -s*k*(1+k)
    e = -s*(1+k)
    if phi == 0:
        def func(x):
            return x
    else:
        def func(x):
            return (-(b*x+e)+sqrt(((b*x+e)**2)-4*c*(a*(x**2)+d*x)))/(2*c)
    integration = quad(func, x1, x2)
    return integration[0]