import iRRAPy.utilRRAgeometry as geo
import iRRAPy.utilRRAareas as ae
import iRRAPy.utilROCgeometry as rocgeo
import iRRAPy.Border as bod

""" Methods used by other function to compute the area or a partial area under
    the ROC curve"""

""" Compute the AUC of a ROC curve"""
def roc_curve_auc(roc_x, roc_y):
    auc = 0
    for i in range(len(roc_x)-1):
        auc += ((roc_y[i]+roc_y[i+1])*(roc_x[i+1]-roc_x[i])/2)
    return auc
    
""" Compute the area between a ROC curve and a given line"""
def roc_curve_area_line(slope, intercept, roc_x, roc_y):
    n = len(roc_x)
    area = 0
    for j in range(n-1):
        x1 = roc_x[j]
        y1 = roc_y[j]
        x2 = roc_x[j+1]
        y2 = roc_y[j+1]
        yc1 = slope*x1 + intercept
        yc2 = slope*x2 + intercept
        if yc1 < 0:
            yc1 = 0
        if yc2 < 0:
            yc2 = 0
        
        if x1 == x2 or (yc1 >= y1 and yc2 >= y2):
            area += 0
        else:
            if yc1 <= y1 and yc2 <= y2:
                area += ae.quadrilateral_area(x1, y1, yc1, x2, y2, yc2)
            else:
                if (y1 >= yc1 and y2 <= yc2) or (y1 <= yc1 and y2 >= yc2):
                    inter = geo.intersect_line_segment(slope, intercept, x1,
                                                    y1, x2, y2)
                    x_int = inter.x
                    y_int = inter.y
                    if y1 >= yc1 and y2 <= yc2:
                        area+=ae.triangle_area(x1, y1, x1, yc1, x_int, y_int)
                    else:
                        area+=ae.triangle_area(x_int, y_int, x2, yc2, x2, y2)
    return area
   
""" Compute the area between a ROC curve and a partial line"""   
def roc_curve_partial_area_line(roc_x, roc_y, slope, intercept, xsx, xdx):
    if xsx < xdx:
        roc = rocgeo.partial_roc_curve(roc_x, roc_y, xsx, xdx)
        vx = roc[0]
        vy = roc[1]
        area = roc_curve_area_line(slope, intercept, vx, vy)
        return area
    else:
        return 0
        
""" Compute the partial AUC limited by a vertical line"""
def roc_curve_vertical_line(roc_x, roc_y, xlim):
    done = False
    if xlim < 0:
        return 0
    elif xlim > 1:
        area = roc_curve_auc(roc_x, roc_y)
        return area
    area = 0
    for i in range(len(roc_x)-1):
        x1 = roc_x[i]
        y1 = roc_y[i]
        if roc_x[i+1] > xlim:
            y2 = geo.intersect_segments(x1, y1, roc_x[i+1], 
                                        roc_y[i+1], xlim, 0, xlim, 1)
            x2 = xlim
            done = True
        else:
            x2 = roc_x[i+1]
            y2 = roc_y[i+1]
        area += ((y2+y1)*(x2-x1)/2)
        if done:
            break
    return area
    
""" Compute the area of a RoI generated by a ROC
    curve and the area under the curve. It returns a list"""    
def aree_phi(phi, k, vx, vy):
    where_y_one = geo.intersect_phi_line(phi, k, 0, 1)
    xt = min(e.x for e in where_y_one)
    nn = len(vx)
    found_x = False
    jj = 0
    while not found_x and jj < nn:
        if vx[jj] >= xt:
            found_x = True
        else:
            jj += 1
    if found_x and jj>0:
        xa = vx[jj-1]
        ya = vy[jj-1]
        xb = vx[jj]
        yb = vy[jj]
        yt = ya+(yb-ya)*(xt-xa)/(xb-xa)
        vx = [*vx[0:jj], xt, *vx[jj:nn-1]]
        vy = [*vy[0:jj], yt, *vy[jj:nn-1]]
    nn = len(vx)
    vphi = bod.phi_border(phi, k, vx)
    vyphi = vphi[1]
    phi_roi = 0
    phi_under = 0
    for j in range(nn-1):
        x1 = vx[j]
        y1 = vy[j]
        x2 = vx[j+1]
        y2 = vy[j+1]
        if x2 > x1:
            yphi1 = vyphi[j]
            yphi2 = vyphi[j+1]
            if (1-yphi2) < 0.000000001:
                yphi2 = 1
            if (1-yphi1) < 0.000000001:
                yphi1 = 1
            if y2 >= yphi2 and y1 >= yphi1:
                if yphi1 == 1 and yphi2 == 1:
                    ttt = (x2-x1)
                else:
                    ttt = ae.area_under_phi_interval(phi, k, x1, x2)
                area_over_phi = (x2-x1)-ttt
                phi_roi += area_over_phi
                if len(geo.intersect_phi_segment(phi, k, x1, x2, y1, y2)) > 1:
                    ttt = geo.intersect_phi_segment(phi, k, x1, x2, y1, y2)
                    xint1 = ttt[0].x
                    yint1 = ttt[0].y
                    xint2 = ttt[1].x
                    yint2 = ttt[1].y
                    ttt = ae.area_under_phi_interval(phi, k, x1, xint1)
                    delta = ((yint1+y1)*(xint1-x1)/2) - ttt
                    delta = max(0,delta)
                    phi_under += delta
                    ttt = ae.area_under_phi_interval(phi, k, xint2, x2)
                    delta = ((yint2+y2)*(x2-xint2)/2) - ttt
                    delta = max(0,delta)
                    phi_under += delta
                else:
                    gArea = ((y2+y1)*(x2-x1)/2) - ttt
                    phi_under += gArea
            else:
                if (y2 > yphi2 and y1 < yphi1) or (y2 < yphi2 and y1 > yphi1):
                    ttt = geo.intersect_phi_segment(phi, k, x1, x2, y1, y2)
                    xint = ttt[0].x
                    yint = ttt[0].y
                    ttt = ae.area_under_phi_interval(phi, k, x1, xint)
                    phi_roi += (xint-x1)-ttt
                    delta = ((yint+y1)*(xint-x1)/2) - ttt
                    delta = max(0,delta)
                    phi_under += delta
                    ttt = ae.area_under_phi_interval(phi, k, xint, x2)
                    phi_roi += (x2-xint) - ttt
                    delta = ((yint+y2)*(x2-xint)/2) - ttt
                    delta = max(0,delta)
                    phi_under += delta
                else:
                    if yphi1 == 1 and yphi2 == 1:
                        ttt = x2-x1
                    else:
                        ttt = ae.area_under_phi_interval(phi, k, x1, x2)
                    area_over_phi = (x2-x1) - ttt
                    phi_roi += area_over_phi
    
    return [phi_roi, phi_under]
    
""" Compute the area of a RoI generated by a ROC
    curve and the area under the curve. It returns a list"""    
def aree_markedness(c_markedness, k, vx, vy):
    where_y_one = geo.intersect_markedness_line(c_markedness, k, 0, 1)
    if len(where_y_one) == 1:
        xt = where_y_one[0].x
    else:
        where_y_one = where_y_one.sort(key = lambda j: j.x)
        xt = where_y_one[0].x
    nn = len(vx)
    found_x = False
    jj = 0
    while not found_x and jj < nn:
        if vx[jj] >= xt:
            found_x = True
        else:
            jj += 1
    if found_x and jj>0:
        xa = vx[jj-1]
        ya = vy[jj-1]
        xb = vx[jj]
        yb = vy[jj]
        yt = ya+(yb-ya)*(xt-xa)/(xb-xa)
        vx = [*vx[0:jj], xt, *vx[jj:nn-1]]
        vy = [*vy[0:jj], yt, *vy[jj:nn-1]]
    nn = len(vx)
    vmark = bod.markedness_border(c_markedness, k ,vx)
    vymark = vmark[1]
    mark_roi = 0
    mark_under = 0
    for j in range(nn-1):
        x1 = vx[j]
        y1 = vy[j]
        x2 = vx[j+1]
        y2 = vy[j+1]
        if x2 > x1:
            ymark1 = vymark[j]
            ymark2 = vymark[j+1]
            if (1-ymark2) < 0.000000001:
                ymark2 = 1
            if (1-ymark1) < 0.000000001:
                ymark1 = 1
            if y2 >= ymark2 and y1 >= ymark1:
                if ymark1 == 1 and ymark2 == 1:
                    ttt = (x2-x1)
                else:
                    ttt = ae.area_under_markedness_interval(c_markedness,
                                                        k, x1, x2)
                mark_roi += (x2-x1) - ttt
                if(len(geo.intersect_markedness_segment(c_markedness, 
                                                    k, x1, x2, y1, y2)) > 1):
                    ttt = geo.intersect_markedness_segment(c_markedness, 
                                                    k, x1, x2, y1, y2)
                    xint1 = ttt[0].x
                    yint1 = ttt[0].y
                    xint2 = ttt[1].x
                    yint2 = ttt[1].y
                    ttt = ae.area_under_markedness_interval(c_markedness,
                                                        k, x1, xint1)
                    delta = ((yint1+y1)*(xint1-x1)/2) - ttt
                    delta = max(0, delta)
                    mark_under += delta
                    ttt = ae.area_under_markedness_interval(c_markedness,
                                                        k, xint2, x2)
                    delta = ((yint2+y2)*(x2-xint2)/2) - ttt
                    delta = max(0,delta)
                    mark_under += delta
                else:
                    mark_under += ((y2+y1)*(x2-x1)/2) - ttt
            else:
                if (y2>ymark2 and y1<ymark1) or (y2<ymark2 and y1>ymark1):
                    ttt = geo.intersect_markedness_segment(c_markedness, k,
                                                            x1, x2, y1, y2)
                    xint = ttt[0].x
                    yint = ttt[0].y
                    ttt = ae.area_under_markedness_interval(c_markedness,
                                                            k, x1, xint)
                    mark_roi += (xint-x1) - ttt
                    delta = ((yint+y1)*(xint-x1)/2) - ttt
                    delta = max(0,delta)
                    mark_under += delta
                    ttt = ae.area_under_markedness_interval(c_markedness,
                                                            k, xint, x2)
                    mark_roi += (x2-xint) - ttt
                    delta = ((yint+y2)*(x2-xint)/2) - ttt
                    delta = max(0,delta)
                    mark_under += delta
                else:
                    if ymark1 == 1 and ymark2 == 1:
                        ttt = x2-x1
                    else:
                        ttt = ae.area_under_markedness_interval(c_markedness,
                                                            k, x1, x2)
                    area_over_mark = (x2-x1) - ttt
                    mark_roi += area_over_phi
    
    return [mark_roi, mark_under]
    
""" Compute the areas of a RoI generated by a phi curve within a 
    certain range"""
def roc_partial_area_phi(roc_x, roc_y, phi, k, xsx, xdx):
    if xsx < xdx:
        roc = rocgeo.partial_roc_curve(roc_x, roc_y, xsx, xdx)
        vx = roc[0]
        vy = roc[1]
        return aree_phi(phi, k, vx, vy)
    else:
        return [0,0]

""" Compute the areas of a RoI generated by a markedness curve within a 
    certain range"""
def roc_partial_area_markedness(roc_x, roc_y, c_markedness, k, xsx, xdx):
    if xsx < xdx:
        roc = rocgeo.partial_roc_curve(roc_x, roc_y, xsx, xdx)
        vx = roc[0]
        vy = roc[1]
        return aree_markedness(c_markedness, k, vx, vy)
    else:
        return [0,0]