from math import sqrt

""" Functions that can be used to find coordinates such as intersections. They
    return NULL when there is no intersections, or at least no interesting
    intersections """
"""Class definition of a intersection point"""
class Intersection:
    def __init__(self, x, y):
        self.x = x
        self.y = y


""" Intersection between two segments"""
def intersect_segments(x11, y11, x12, y12, x21, y21, x22, y22):
    is_vertical1 = False
    is_vertical2 = False
    if x11 != x12:
        slope1 = (y12-y11)/(x12-x11)
        intercept1 = y11-((x11*(y12-y11))/(x12-x11))
    else:
        slope1 = "none1"
        is_vertical1 = True
    if x21 != x22:
        slope2 = (y22-y21)/(x22-x21)
        intercept2 = y21-((x21*(y22-y21))/(x22-x21))
    else:
        slope2 = "none2"
        is_vertical2 = True

    if ((is_vertical1 and is_vertical2) or slope1 == slope2):
        return []
    if not(is_vertical1 or is_vertical2):
        point = intersect_lines(slope1, intercept1, slope2, intercept2)
        if point.x >= min(x11,x12) and point.x <= max(x11,x12):
            return point
        else:
            return []
    else:
        if is_vertical1:
            point = intersect_line_vertical_line(slope2, intercept2, x11)
            if x11 >= min(x21,x22) and x11 <= max(x21,x22):
                if point >= min(y11,y12) and point <= max(y11,y12):
                    return point
                else:
                    return []
            else:
                return []
        else:
            point = intersect_line_vertical_line(slope1, intercept1, x21)     
            if x21 >= min(x11,x12) and x21 <= max(x11,x12):
                if point >= min(y21,y22) and point <= max(y21,y22):
                    return point
                else:
                    return []
            else:
                return []
        
"""Intersection between a line and a segment"""
def intersect_line_segment(slope1, intercept1, x1, y1, x2, y2):
    is_vertical2 = False
    if x1 != x2:
        slope2 = (y2-y1)/(x2-x1)
        intercept2 = y1-((x1*(y2-y1))/(x2-x1))
    else:
        is_vertical2 = True

    if is_vertical2:
        vx = x1
        vy = intersect_line_vertical_line(slope1, intercept1, x1)
        if vy < min(y1,y2) or vy > max(y1,y2):
            return []
        else:
            result = Intersection(vx, vy)
            return result
    else:
        point = intersect_lines(slope1, intercept1, slope2, intercept2)
        if point.x >= min(x1,x2) and point.x <= max(x1,x2):
            return point
        else:
            return []
            
"""Intersection between a vertical line and a segment. It returns only
    the y value"""
def intersect_vertical_line_segment(x, x1, y1, x2, y2):
    if (x1 != x2):
        slope2 = (y2-y1)/(x2-x1)
        intercept2 = y1-((x1*(y2-y1))/(x2-x1))
    else:
        return []
    if x >= min(x1,x2) and x <= max(x1,x2):
        return intersect_line_vertical_line(slope2, intercept2, x)
    else:
        return []

"""Intersection between two lines"""
def intersect_lines(slope1, intercept1, slope2, intercept2):
    if slope1 == slope2:
        return []
    x12 = (intercept2-intercept1)/(slope1-slope2)
    y12 = slope1*x12 + intercept1
    result = Intersection(x12, y12)
    return result

"""Intersection between a line and a vertical line. It returns
    only the y value"""
def intersect_line_vertical_line(slope1, intercept1, x):
    y = slope1*x + intercept1
    return y
    
"""Intersection between a vertical line and a phi curve. It only returns
    the y value"""
def intersect_phi_vertical_line(phi, k, x):
    p = phi**2
    a = p+k
    b = (2*k*(p-1)*x)-(p*(k+1))
    c = (k*(x**2)*(1+p*k))-(p*k*x*(k+1))
    if a == 0 or (b**2) < 4*a*c:
        return None
    else:
        yr = (-b+sqrt((b**2)-4*a*c))/(2*a)
        return yr

"""Intersection between a line and a phi curve. It returns a list of
    Intersection objects"""
def intersect_phi_line(phi, k, slope, intercept):
    p = phi**2
    m = slope
    q = intercept
    
    a = k*(1+p*k) + m*(k*(2*p-2+m)+p*m)
    b = 2*q*(k*(p+m-1)+p*m) - p*(k+1)*(k+m)
    c = q*(p*(q-k-1)+k*q)
    
    if a == 0 or (b**2) < 4*a*c:
        return []
    else:
        x1 = (-b-sqrt((b**2)-4*a*c))/(2*a)
        x2 = (-b+sqrt((b**2)-4*a*c))/(2*a)
        y1 = m*x1 + q
        y2 = m*x2 + q
        
        result = []
        if y1 >= x1 and x1 >= 0:
            tmp = Intersection(x1, y1)
            result.append(tmp)
        if y2 >= x2 and x2 >= 0:
            tmp = Intersection(x2, y2)
            result.append(tmp)
        if len(result) == 0:
            if slope == 0 and intercept == 1:
                result.append(Intersection(1,1))
        return result
            
"""Intersection between a segment and a phi curve. It returns a list of
    Intersection objects."""
def intersect_phi_segment(phi, k, x1, x2, y1, y2):
    is_vertical2 = False
    if x1 != x2:
        slope2 = (y2-y1)/(x2-x1)
        intercept2 = y1-((x1*(y2-y1))/(x2-x1))
    else:
        is_vertical2 = True
        
    if is_vertical2:
        point = intersect_phi_vertical_line(phi, k, x2)
        if point < min(y1,y2) or point > max(y1,y2):
            return []
        else:
            result = [Intersection(x1, point)]
            return result
    else:
        point = intersect_phi_line(phi, k, slope2, intercept2)
        if len(point) <= 1:
            if (len(point) == 1 and point[0].x >= min(x1,x2) and
                                        point[0].x <= max(x1,x2)):
                return point
            else:
                return []
        else:
            result = []
            xf = point[0].x
            xl = point[1].x
            if xf >= min(x1,x2) and xf <= max(x1,x2):
                result.append(point[0])
            if xl >= min(x1,x2) and xl <= max(x1,x2):
                result.append(point[1])
            return result

"""Intersection between a vertical line and a markedness curve. 
    It only returns the y value"""
def intersect_markedness_vertical_line(c_markedness, k, x):
    a = c_markedness
    b = 2*c_markedness*k*x - c_markedness*k + k - c_markedness
    c = k*x*(c_markedness*k*x - (c_markedness*k + c_markedness + 1))
    if a == 0 or (b**2) < 4*a*c:
        return None
    else:
        yr = (-b+sqrt((b**2)-4*a*c))/(2*a)
        return yr

"""Intersection between a line and a markedness curve. It returns a list
    of Intersection objects"""
def intersect_markedness_line(c_markedness, k, slope, intercept):
    p = c_markedness
    m = slope
    q = intercept
    
    a = p*(k + m)*(k + m)
    b = k*(p*(2*q -k - m - 1) + m - 1) + m*p*(2*q - 1)
    c = q*(p*(q - k - 1) + k)
    
    if a == 0 or (b**2) < 4*a*c:
        return []
    else:
        x1 = (-b-sqrt((b**2)-4*a*c))/(2*a)
        x2 = (-b+sqrt((b**2)-4*a*c))/(2*a)
        y1 = m*x1 + q
        y2 = m*x2 + q
        result = []
        if y1 >= x1 and x1 >= 0:
            tmp = Intersection(x1, y1)
            result.append(tmp)
        if y2 >= x2 and x2 >= 0:
            tmp = Intersection(x2, y2)
            result.append(tmp)
        if len(result) == 0:
            if slope == 0 and intercept == 1:
                result.append(Intersection(1,1))
        return result

"""Intersection between a segment and a markedness curve. It returns a 
    list of Intersection objects."""
def intersect_markedness_segment(c_markedness, k, x1, x2, y1, y2):
    is_vertical2 = False
    if x1 != x2:
        slope2 = (y2-y1)/(x2-x1)
        intercept2 = y1-((x1*(y2-y1))/(x2-x1))
    else:
        is_vertical2 = True
    
    if is_vertical2:
        point = intersect_markedness_vertical_line(c_markedness, k, x2)
        if point < min(y1,y2) or point > max(y1,y2):
            return []
        else:
            result = [Intersection(x1, point)]
            return result
    else:
        point = intersect_markedness_line(c_markedness, k, slope2, intercept2)
        if len(point) <= 1:
            if (len(point) == 1 and point[0].x >= min(x1,x2) and 
                                                point[0].x <= max(x1,x2)):
                return point
            else:
                return []
        else:
            result = []
            xf = point[0].x
            xl = point[1].x
            if xf >= min(x1,x2) and xf <= max(x1,x2):
                result.append(point[0])
            if xl >= min(x1,x2) and xl <= max(x1,x2):
                result.append(point[1])
            return result

"""Intersection between a phi curve and a markedness curve"""
def intersect_phi_markedness(phi, c_markedness, k):
    q = (phi**2)/c_markedness
    result = intersect_phi_line(phi, k, 1, q)
    return result
    
    
